<?php

namespace Drupal\simple_sitemap_language_hierarchy;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\Cache;

/**
 * Class LanguageHierarchyManager.
 *
 * @package Drupal\simple_sitemap_language_hierarchy
 */
class SimpleSitemapLanguageHierarchyManager {

  /**
   * Language Hierarchy array Cache.
   *
   * @var string
   */
  const LANGUAGE_HIERARCHY_CACHE_NAME = 'simple_sitemap_language_hierarchy:array';

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language storage.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs a TranslationManager object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_manager
   *   Entity type manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Drupal cache.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_manager,
    CacheBackendInterface $cache
  ) {
    $this->entityTypeManager = $entity_manager;
    $this->cache = $cache;
  }

  /**
   * Get all language parent.
   */
  public function getAllParentLanguage($langCode, $languageConfigEntities = FALSE) {

    // Check lang code.
    if (empty($langCode)) {
      return FALSE;
    }

    $result = &drupal_static(__FUNCTION__);

    if (!empty($result[$langCode])) {
      return $result[$langCode];
    }

    // Initial value.
    $result[$langCode] = [];

    // Get fallback language.
    $languageFallback = $this->getLanguageFallback($langCode, $languageConfigEntities);

    if (empty($languageFallback)) {
      return $result[$langCode];
    }

    $result[$langCode][] = $languageFallback;

    $currentlangCode = $languageFallback;

    while (!empty($languageFallback)) {

      $languageFallback = $this->getLanguageFallback($currentlangCode, $languageConfigEntities);

      if (empty($languageFallback)) {
        return $result[$langCode];
      }

      $result[$langCode][] = $languageFallback;

      $currentlangCode = $languageFallback;
    }

    return $result[$langCode];
  }

  /**
   * Get Languages Fallback.
   */
  public function getLanguageFallback($langCode, $languageConfigEntities = FALSE) {

    if (empty($langCode)) {
      return FALSE;
    }

    $langCodeFallbackParents = &drupal_static(__FUNCTION__);

    if (!empty($langCodeFallbackParents[$langCode])) {
      return $langCodeFallbackParents[$langCode];
    }

    $configManager = $this->entityTypeManager->getStorage('configurable_language');

    // If we already load this language config entity, use it. Else load a new.
    if (!empty($languageConfigEntities[$langCode])) {
      $languageConfigEntity = $languageConfigEntities[$langCode];
    }
    else {
      $languageConfigEntity = $configManager->load($langCode);
    }

    $langCodeFallbackParents[$langCode] = FALSE;
    if (!empty($languageConfigEntity->getThirdPartySetting('language_hierarchy', 'fallback_langcode', ''))) {
      $langCodeFallbackParents[$langCode] = $languageConfigEntity->getThirdPartySetting('language_hierarchy', 'fallback_langcode', '');
    }

    return $langCodeFallbackParents[$langCode];
  }

  /**
   * Get all language hierarchy saved on cache.
   *
   * @return array
   *   All language hierarchy saved on cache.
   */
  public function getCachedHierarchyArray() {
    $data = [];
    if ($cache = $this->cache
      ->get(self::LANGUAGE_HIERARCHY_CACHE_NAME)
    ) {
      return $cache->data;
    }

    $data = language_hierarchy_tree();

    $tags = [];
    foreach ($data as $langCode => &$languageData) {
      $tags[] = "hierarchy_array:{$langCode}";

      $configurableLanguage = $languageData['#data'];
      $providers = $configurableLanguage->getThirdPartyProviders();
      $languageData['#settings'] = [];
      foreach ($providers as $provider) {
        $languageData['#settings'] += $configurableLanguage->getThirdPartySettings($provider);
      }

    }

    $this->cache
      ->set(self::LANGUAGE_HIERARCHY_CACHE_NAME, $data, Cache::PERMANENT, $tags);

    return $data;
  }

}
